@@ -387,6 +387,14 @@ lav_matrix_vech_match_idx <- function(n = 1L, diagonal = TRUE,
 # default dup:
 lav_matrix_duplication <- .dup3
 
+# duplication matrix for correlation matrices:
+# - it returns a matrix of size p^2 * (p*(p-1))/2
+# - the columns corresponding to the diagonal elements have been removed
+lav_matrix_duplication_cor <- function(n = 1L) {
+    out <- lav_matrix_duplication(n = n)
+    diag.idx <- lav_matrix_diagh_idx(n = n)
+    out[,-diag.idx,drop = FALSE]
+}
 
 # compute t(D) %*% A (without explicitly computing D)
 # sqrt(nrow(A)) is an integer
@@ -432,6 +440,30 @@ lav_matrix_duplication_dup_pre2 <- function(A = matrix(0,0,0)) {
     OUT
 }
 
+# compute t(D) %*% A (without explicitly computing D)
+# sqrt(nrow(A)) is an integer
+# A is not symmetric, and not even square, only n^2 ROWS
+# correlation version: ignoring diagonal elements
+lav_matrix_duplication_cor_pre <- function(A = matrix(0,0,0)) {
+
+    # number of rows
+    n2 <- NROW(A)
+
+    # square nrow(A) only, n2 = n^2
+    stopifnot(sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)
+
+    # dup idx
+    idx1 <- lav_matrix_vech_idx(n,   diagonal = FALSE)
+    idx2 <- lav_matrix_vechru_idx(n, diagonal = FALSE)
+
+    OUT <- A[idx1, , drop = FALSE] + A[idx2 , , drop = FALSE]
+    u <- which(idx1 %in% idx2); OUT[u,] <- OUT[u,] / 2.0
+
+    OUT
+}
 
 # compute A %*% D (without explicitly computing D)
 # sqrt(ncol(A)) must be an integer
@@ -456,6 +488,32 @@ lav_matrix_duplication_post <- function(A = matrix(0,0,0)) {
     OUT
 }
 
+# compute A %*% D (without explicitly computing D)
+# sqrt(ncol(A)) must be an integer
+# A is not symmetric, and not even square, only n^2 COLUMNS
+# correlation version: ignoring the diagonal elements
+lav_matrix_duplication_cor_post <- function(A = matrix(0,0,0)) {
+
+    # number of columns
+    n2 <- NCOL(A)
+
+    # square A only, n2 = n^2
+    stopifnot(sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)
+
+    # dup idx
+    idx1 <- lav_matrix_vech_idx(n,   diagonal = FALSE)
+    idx2 <- lav_matrix_vechru_idx(n, diagonal = FALSE)
+
+    OUT <- A[, idx1, drop = FALSE] + A[, idx2, drop = FALSE]
+    u <- which(idx1 %in% idx2); OUT[,u] <- OUT[,u] / 2.0
+
+    OUT
+}
+
+
 # compute t(D) %*% A %*% D (without explicitly computing D)
 # A must be a square matrix and sqrt(ncol) an integer
 lav_matrix_duplication_pre_post <- function(A = matrix(0,0,0)) {
@@ -480,6 +538,32 @@ lav_matrix_duplication_pre_post <- function(A = matrix(0,0,0)) {
     OUT
 }
 
+# compute t(D) %*% A %*% D (without explicitly computing D)
+# A must be a square matrix and sqrt(ncol) an integer
+# correlation version: ignoring diagonal elements
+lav_matrix_duplication_cor_pre_post <- function(A = matrix(0,0,0)) {
+
+    # number of columns
+    n2 <- NCOL(A)
+
+    # square A only, n2 = n^2
+    stopifnot(NROW(A) == n2, sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)
+
+    # dup idx
+    idx1 <- lav_matrix_vech_idx(n,   diagonal = FALSE)
+    idx2 <- lav_matrix_vechru_idx(n, diagonal = FALSE)
+
+    OUT <- A[idx1, , drop = FALSE] + A[idx2, , drop = FALSE]
+    u <- which(idx1 %in% idx2); OUT[u,] <- OUT[u,] / 2.0
+    OUT <- OUT[, idx1, drop = FALSE] + OUT[, idx2, drop = FALSE]
+    OUT[,u] <- OUT[,u] / 2.0
+
+    OUT
+}
+
 # create the generalized inverse of the duplication matrix (D^+_n):
 # it removes the duplicated elements in vec(S) to create vech(S)
 #
@@ -657,8 +741,10 @@ lav_matrix_commutation <- .com1
 # = permuting the rows of A
 lav_matrix_commutation_pre <- function(A = matrix(0,0,0)) {
 
+    A <- as.matrix(A)
+
     # number of rows of A
-    n2 <- NROW(A)
+    n2 <- nrow(A)
 
     # K_nn only (n2 = m * n)
     stopifnot(sqrt(n2) == round(sqrt(n2)))
@@ -674,6 +760,56 @@ lav_matrix_commutation_pre <- function(A = matrix(0,0,0)) {
     OUT
 }
 
+# compute A %*% K_n without explicitly computing K
+# K_n = K_nn, so sqrt(ncol(A)) must be an integer!
+# = permuting the columns of A
+lav_matrix_commutation_post <- function(A = matrix(0,0,0)) {
+
+    A <- as.matrix(A)
+
+    # number of columns of A
+    n2 <- ncol(A)
+     
+    # K_nn only (n2 = m * n)
+    stopifnot(sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)
+    
+    # compute col indices
+    #row.idx <- as.integer(t(matrix(1:n2, n, n)))
+    col.idx <- rep(1:n, each = n) + (0:(n-1L))*n
+
+    OUT <- A[, col.idx, drop = FALSE]
+    OUT 
+}
+
+# compute K_n %*% A %*% K_n without explicitly computing K
+# K_n = K_nn, so sqrt(ncol(A)) must be an integer!
+# = permuting both the rows AND columns of A
+lav_matrix_commutation_pre_post <- function(A = matrix(0,0,0)) {
+
+    A <- as.matrix(A)
+
+    # number of columns of A
+    n2 <- NCOL(A)
+    
+    # K_nn only (n2 = m * n)
+    stopifnot(sqrt(n2) == round(sqrt(n2)))
+
+    # dimension
+    n <- sqrt(n2)     
+   
+    # compute col indices
+    row.idx <- rep(1:n, each = n) + (0:(n-1L))*n
+    col.idx <- row.idx
+
+    OUT <- A[row.idx, col.idx, drop = FALSE]
+    OUT
+}
+
+
+
 # compute K_mn %*% A without explicitly computing K
 # = permuting the rows of A
 lav_matrix_commutation_mn_pre <- function(A, m = 1L, n = 1L) {
@@ -1074,7 +1210,7 @@ lav_matrix_symmetric_inverse <- function(S, logdet   = FALSE,
         if(zero.warn) {
             warning("lavaan WARNING: matrix to be inverted contains zero cols/rows")
         }
-        S <- S[-zero.idx, -zero.idx]
+        S <- S[-zero.idx, -zero.idx, drop = FALSE]
     }
 
     P <- NCOL(S)
