@@ -50,7 +50,9 @@ print.lavaan.matrix.symmetric <- function(x, ..., nd = 3L, shift = 0L,
     # print only lower triangle of a symmetric matrix
     # this function was inspired by the `print.correlation' function
     # in package nlme
+    x <- as.matrix(x) # just in case
     y <- x; y <- unclass(y)
+    attributes(y)[c("header","footer")] <- NULL
     ll <- lower.tri(x, diag = TRUE)
     y[ ll] <- format(round(x[ll], digits = nd))
     y[!ll] <- ""
@@ -73,13 +75,25 @@ print.lavaan.matrix.symmetric <- function(x, ..., nd = 3L, shift = 0L,
             rownames(y) <- empty.string
         }
     }
+
+    if(!is.null(attr(x, "header"))) {
+      cat("\n", attr(x, "header"), "\n\n", sep = "")
+    }
+
     print(y, ..., quote = FALSE, right = TRUE)
+
+    if(!is.null(attr(x, "footer"))) {
+      cat("\n", attr(x, "footer"), "\n\n", sep = "")
+    }
+
     invisible(x)
 }
 
 
 print.lavaan.matrix <- function(x, ..., nd = 3L, shift = 0L) {
+    x <- as.matrix(x) # just in case
     y <- unclass(x)
+    attributes(y)[c("header","footer")] <- NULL
     if(!is.null(colnames(x))) {
       colnames(y) <- abbreviate(colnames(x), minlength = nd + 3L)
     }
@@ -91,15 +105,29 @@ print.lavaan.matrix <- function(x, ..., nd = 3L, shift = 0L) {
             rownames(y) <- empty.string
         }
     }
+    if(!is.null(attr(x, "header"))) {
+      cat("\n", attr(x, "header"), "\n\n", sep = "")
+    }
+
     print( round(y, nd), right = TRUE, ... )
+
+    if(!is.null(attr(x, "footer"))) {
+      cat("\n", attr(x, "footer"), "\n\n", sep = "")
+    }
+
     invisible(x)
 }
 
 print.lavaan.vector <- function(x, ..., nd = 3L, shift = 0L) {
     y <- unclass(x)
+    attributes(y)[c("header","footer")] <- NULL
     #if(!is.null(names(x))) {
     #    names(y) <- abbreviate(names(x), minlength = nd + 3)
     #}
+    if(!is.null(attr(x, "header"))) {
+      cat("\n", attr(x, "header"), "\n\n", sep = "")
+    }
+
     if(shift > 0L) {
         empty.string <- strrep(x = " ", times = shift)
         tmp <- format(y, digits = nd, width = 2L + nd)
@@ -108,6 +136,11 @@ print.lavaan.vector <- function(x, ..., nd = 3L, shift = 0L) {
     } else {
         print( round(y, nd), right = TRUE, ... )
     }
+
+    if(!is.null(attr(x, "footer"))) {
+      cat("\n", attr(x, "footer"), "\n\n", sep = "")
+    }
+
     invisible(x)
 }
 
